@@ -34,6 +34,18 @@
 
 G_DEFINE_TYPE (EBookBackendCouchDB, e_book_backend_couchdb, E_TYPE_BOOK_BACKEND);
 
+static void
+get_current_time (gchar time_string[100])
+{
+	const struct tm *tm = NULL;
+	time_t t;
+
+	t = time (NULL);
+	tm = gmtime (&t);
+	if (tm)
+		strftime (time_string, 100, "%Y-%m-%dT%H:%M:%SZ", tm);
+}
+
 static char *
 vcard_from_couch_document (CouchdbDocument *document)
 {
@@ -42,12 +54,13 @@ vcard_from_couch_document (CouchdbDocument *document)
 	GSList *list, *sl;
 	GList *attr_list;
 	CouchdbStructField *app_annotations;
+	EContactName contact_name;
 
-	if (!couchdb_document_is_contact (document))
+	if (!desktopcouch_document_is_contact (document))
 		return NULL;
 
 	/* Check if the contact is marked for deletion */
-	if ((app_annotations = couchdb_document_get_application_annotations (document))) {
+	if ((app_annotations = desktopcouch_document_get_application_annotations (document))) {
 		CouchdbStructField *u1_annotations;
 
 		u1_annotations = couchdb_struct_field_get_struct_field (
@@ -60,6 +73,7 @@ vcard_from_couch_document (CouchdbDocument *document)
 			if (private_annotations != NULL) {
 				if (couchdb_struct_field_has_field (private_annotations, "deleted")
 				    && couchdb_struct_field_get_boolean_field (private_annotations, "deleted"))
+					couchdb_struct_field_unref (app_annotations);
 					return NULL;
 			}
 		}
@@ -72,43 +86,51 @@ vcard_from_couch_document (CouchdbDocument *document)
 					  couchdb_document_get_revision (document));
 
 	e_contact_set (contact, E_CONTACT_UID, (const gpointer) couchdb_document_get_id (document));
-	e_contact_set (contact, E_CONTACT_GIVEN_NAME,
-		       (const gpointer) couchdb_document_contact_get_first_name (document));
-	e_contact_set (contact, E_CONTACT_FAMILY_NAME,
-		       (const gpointer) couchdb_document_contact_get_last_name (document));
+
+	contact_name.family = desktopcouch_document_contact_get_last_name (document);
+	contact_name.given = desktopcouch_document_contact_get_first_name (document);
+	contact_name.additional = desktopcouch_document_contact_get_middle_name (document);
+	contact_name.prefixes = desktopcouch_document_contact_get_title (document);
+	contact_name.suffixes = desktopcouch_document_contact_get_suffix (document);
+	e_contact_set (contact, E_CONTACT_NAME, (const gpointer) &contact_name);
+
+	str = e_contact_name_to_string (&contact_name);
+	e_contact_set (contact, E_CONTACT_FULL_NAME, (const gpointer) str);
+	g_free (str);
+
 	e_contact_set (contact, E_CONTACT_NICKNAME,
-		       (const gpointer) couchdb_document_contact_get_nick_name (document));
+		       (const gpointer) desktopcouch_document_contact_get_nick_name (document));
 	e_contact_set (contact, E_CONTACT_SPOUSE,
-		       (const gpointer) couchdb_document_contact_get_spouse_name (document));
+		       (const gpointer) desktopcouch_document_contact_get_spouse_name (document));
 
 	e_contact_set (contact, E_CONTACT_ORG,
-		       (const gpointer) couchdb_document_contact_get_company (document));
+		       (const gpointer) desktopcouch_document_contact_get_company (document));
 	e_contact_set (contact, E_CONTACT_ORG_UNIT,
-		       (const gpointer) couchdb_document_contact_get_department (document));
+		       (const gpointer) desktopcouch_document_contact_get_department (document));
 	e_contact_set (contact, E_CONTACT_TITLE,
-		       (const gpointer) couchdb_document_contact_get_title (document));
-	e_contact_set (contact, E_CONTACT_ROLE,
-		       (const gpointer) couchdb_document_contact_get_job_title (document));
+		       (const gpointer) desktopcouch_document_contact_get_job_title (document));
 	e_contact_set (contact, E_CONTACT_MANAGER,
-		       (const gpointer) couchdb_document_contact_get_manager_name (document));
+		       (const gpointer) desktopcouch_document_contact_get_manager_name (document));
 	e_contact_set (contact, E_CONTACT_ASSISTANT,
-		       (const gpointer) couchdb_document_contact_get_assistant_name (document));
+		       (const gpointer) desktopcouch_document_contact_get_assistant_name (document));
 	e_contact_set (contact, E_CONTACT_OFFICE,
-		       (const gpointer) couchdb_document_contact_get_office (document));
+		       (const gpointer) desktopcouch_document_contact_get_office (document));
+	e_contact_set (contact, E_CONTACT_CATEGORIES,
+		       (const gpointer) desktopcouch_document_contact_get_categories (document));
 	e_contact_set (contact, E_CONTACT_NOTE,
-		       (const gpointer) couchdb_document_contact_get_notes (document));
+		       (const gpointer) desktopcouch_document_contact_get_notes (document));
 
 	/* parse email addresses */
 	attr_list = NULL;
 
-	list = couchdb_document_contact_get_email_addresses (document);
+	list = desktopcouch_document_contact_get_email_addresses (document);
 	while (list != NULL) {
 		const char *email_str, *description_str, *uuid_str;
 		EVCardAttribute *attr;
 		CouchdbStructField *email_address = (CouchdbStructField *) list->data;
 
-		email_str = couchdb_document_contact_email_get_address (email_address);
-		description_str = couchdb_document_contact_email_get_description (email_address);
+		email_str = desktopcouch_document_contact_email_get_address (email_address);
+		description_str = desktopcouch_document_contact_email_get_description (email_address);
 		uuid_str = couchdb_struct_field_get_uuid (email_address);
 
 		attr = e_vcard_attribute_new (NULL, e_contact_vcard_attribute (E_CONTACT_EMAIL));
@@ -147,14 +169,14 @@ vcard_from_couch_document (CouchdbDocument *document)
 	}
 
 	/* parse phone numbers */
-	list = couchdb_document_contact_get_phone_numbers (document);
+	list = desktopcouch_document_contact_get_phone_numbers (document);
 	while (list != NULL) {
 		const char *phone_str, *description_str, *uuid_str;
 		EVCardAttribute *attr;
 		CouchdbStructField *phone_number = (CouchdbStructField *) list->data;
 
-		phone_str = couchdb_document_contact_phone_get_number (phone_number);
-		description_str = couchdb_document_contact_phone_get_description (phone_number);
+		phone_str = desktopcouch_document_contact_phone_get_number (phone_number);
+		description_str = desktopcouch_document_contact_phone_get_description (phone_number);
 		uuid_str = couchdb_struct_field_get_uuid (phone_number);
 
 		if (description_str != NULL) {
@@ -259,7 +281,7 @@ vcard_from_couch_document (CouchdbDocument *document)
 	}
 
 	/* parse postal addresses */
-	list = couchdb_document_contact_get_addresses (document);
+	list = desktopcouch_document_contact_get_addresses (document);
 	while (list != NULL) {
 		char **street_lines;
 		const char *description_str;
@@ -269,22 +291,22 @@ vcard_from_couch_document (CouchdbDocument *document)
 		contact_address = g_new0 (EContactAddress, 1);
 		contact_address->address_format = g_strdup ("");
 
-		street_lines = g_strsplit (couchdb_document_contact_address_get_street (address), "\n", 2);
+		street_lines = g_strsplit (desktopcouch_document_contact_address_get_street (address), "\n", 2);
 		if (street_lines != NULL) {
 			contact_address->street = g_strdup (street_lines[0]);
 			if (street_lines[1] != NULL)
 				contact_address->ext = g_strdup (street_lines[1]);
 			g_strfreev (street_lines);
 		} else
-			contact_address->street = g_strdup (couchdb_document_contact_address_get_street (address));
+			contact_address->street = g_strdup (desktopcouch_document_contact_address_get_street (address));
 
-		contact_address->locality = g_strdup (couchdb_document_contact_address_get_city (address));
-		contact_address->region = g_strdup (couchdb_document_contact_address_get_state (address));
-		contact_address->country = g_strdup (couchdb_document_contact_address_get_country (address));
-		contact_address->code = g_strdup (couchdb_document_contact_address_get_postalcode (address));
-		contact_address->po = g_strdup (couchdb_document_contact_address_get_pobox (address));
+		contact_address->locality = g_strdup (desktopcouch_document_contact_address_get_city (address));
+		contact_address->region = g_strdup (desktopcouch_document_contact_address_get_state (address));
+		contact_address->country = g_strdup (desktopcouch_document_contact_address_get_country (address));
+		contact_address->code = g_strdup (desktopcouch_document_contact_address_get_postalcode (address));
+		contact_address->po = g_strdup (desktopcouch_document_contact_address_get_pobox (address));
 		
-		description_str = couchdb_document_contact_address_get_description (address);
+		description_str = desktopcouch_document_contact_address_get_description (address);
 		if (!g_ascii_strcasecmp (description_str, "home"))
 			e_contact_set (contact, E_CONTACT_ADDRESS_HOME, (const gpointer) contact_address);
 		else if (!g_ascii_strcasecmp (description_str, "work"))
@@ -300,14 +322,14 @@ vcard_from_couch_document (CouchdbDocument *document)
 	}
 
 	/* parse URLs */
-	list = couchdb_document_contact_get_urls (document);
+	list = desktopcouch_document_contact_get_urls (document);
 	while (list != NULL) {
 		const char *description_str, *address_str, *uuid_str;
 		EVCardAttribute *attr;
 		CouchdbStructField *url = (CouchdbStructField *) list->data;
 
-		address_str = couchdb_document_contact_url_get_address (url);
-		description_str = couchdb_document_contact_url_get_description (url);
+		address_str = desktopcouch_document_contact_url_get_address (url);
+		description_str = desktopcouch_document_contact_url_get_description (url);
 		uuid_str = couchdb_struct_field_get_uuid (url);
 
 		if (description_str != NULL) {
@@ -334,15 +356,15 @@ vcard_from_couch_document (CouchdbDocument *document)
 	}
 
 	/* parse IM addresses */
-	list = couchdb_document_contact_get_im_addresses (document);
+	list = desktopcouch_document_contact_get_im_addresses (document);
 	while (list != NULL) {
 		const char *address_str, *description_str, *protocol_str, *uuid_str;
 		EVCardAttribute *attr = NULL;
 		CouchdbStructField *im = (CouchdbStructField *) list->data;
 
-		address_str = couchdb_document_contact_im_get_address (im);
-		description_str = couchdb_document_contact_im_get_description (im);
-		protocol_str = couchdb_document_contact_im_get_protocol (im);
+		address_str = desktopcouch_document_contact_im_get_address (im);
+		description_str = desktopcouch_document_contact_im_get_description (im);
+		protocol_str = desktopcouch_document_contact_im_get_protocol (im);
 		/* Some records don't have the 'protocol' field, and use the
 		   'description' field to specify the kind of IM address this
 		   refers to */
@@ -351,21 +373,21 @@ vcard_from_couch_document (CouchdbDocument *document)
 		uuid_str = couchdb_struct_field_get_uuid (im);
 
 		if (protocol_str != NULL) {
-			if (g_strcmp0 (protocol_str, COUCHDB_DOCUMENT_CONTACT_IM_PROTOCOL_AIM) == 0)
+			if (g_strcmp0 (protocol_str, DESKTOPCOUCH_DOCUMENT_CONTACT_IM_PROTOCOL_AIM) == 0)
 				attr = e_vcard_attribute_new (NULL, e_contact_vcard_attribute (E_CONTACT_IM_AIM));
-			else if (g_strcmp0 (protocol_str, COUCHDB_DOCUMENT_CONTACT_IM_PROTOCOL_GADU_GADU) == 0)
+			else if (g_strcmp0 (protocol_str, DESKTOPCOUCH_DOCUMENT_CONTACT_IM_PROTOCOL_GADU_GADU) == 0)
 				attr = e_vcard_attribute_new (NULL, e_contact_vcard_attribute (E_CONTACT_IM_GADUGADU));
-			else if (g_strcmp0 (protocol_str, COUCHDB_DOCUMENT_CONTACT_IM_PROTOCOL_GROUPWISE) == 0)
+			else if (g_strcmp0 (protocol_str, DESKTOPCOUCH_DOCUMENT_CONTACT_IM_PROTOCOL_GROUPWISE) == 0)
 				attr = e_vcard_attribute_new (NULL, e_contact_vcard_attribute (E_CONTACT_IM_GROUPWISE));
-			else if (g_strcmp0 (protocol_str, COUCHDB_DOCUMENT_CONTACT_IM_PROTOCOL_ICQ) == 0)
+			else if (g_strcmp0 (protocol_str, DESKTOPCOUCH_DOCUMENT_CONTACT_IM_PROTOCOL_ICQ) == 0)
 				attr = e_vcard_attribute_new (NULL, e_contact_vcard_attribute (E_CONTACT_IM_ICQ));
-			else if (g_strcmp0 (protocol_str, COUCHDB_DOCUMENT_CONTACT_IM_PROTOCOL_JABBER) == 0)
+			else if (g_strcmp0 (protocol_str, DESKTOPCOUCH_DOCUMENT_CONTACT_IM_PROTOCOL_JABBER) == 0)
 				attr = e_vcard_attribute_new (NULL, e_contact_vcard_attribute (E_CONTACT_IM_JABBER));
-			else if (g_strcmp0 (protocol_str, COUCHDB_DOCUMENT_CONTACT_IM_PROTOCOL_MSN) == 0)
+			else if (g_strcmp0 (protocol_str, DESKTOPCOUCH_DOCUMENT_CONTACT_IM_PROTOCOL_MSN) == 0)
 				attr = e_vcard_attribute_new (NULL, e_contact_vcard_attribute (E_CONTACT_IM_MSN));
-			else if (g_strcmp0 (protocol_str, COUCHDB_DOCUMENT_CONTACT_IM_PROTOCOL_SKYPE) == 0)
+			else if (g_strcmp0 (protocol_str, DESKTOPCOUCH_DOCUMENT_CONTACT_IM_PROTOCOL_SKYPE) == 0)
 				attr = e_vcard_attribute_new (NULL, e_contact_vcard_attribute (E_CONTACT_IM_SKYPE));
-			else if (g_strcmp0 (protocol_str, COUCHDB_DOCUMENT_CONTACT_IM_PROTOCOL_YAHOO) == 0)
+			else if (g_strcmp0 (protocol_str, DESKTOPCOUCH_DOCUMENT_CONTACT_IM_PROTOCOL_YAHOO) == 0)
 				attr = e_vcard_attribute_new (NULL, e_contact_vcard_attribute (E_CONTACT_IM_YAHOO));
 
 			if (attr != NULL) {
@@ -399,7 +421,7 @@ vcard_from_couch_document (CouchdbDocument *document)
 	}
 
 	/* birth date */
-	str = (char *) couchdb_document_contact_get_birth_date (document);
+	str = (char *) desktopcouch_document_contact_get_birth_date (document);
 	if (str && strlen (str) > 0) {
 		EContactDate *dt;
 
@@ -411,7 +433,7 @@ vcard_from_couch_document (CouchdbDocument *document)
 	}
 
 	/* wedding date */
-	str = (char *) couchdb_document_contact_get_wedding_date (document);
+	str = (char *) desktopcouch_document_contact_get_wedding_date (document);
 	if (str && strlen (str)) {
 		EContactDate *dt;
 
@@ -423,16 +445,39 @@ vcard_from_couch_document (CouchdbDocument *document)
 	}
 
 	/* application annotations */
-	if (couchdb_document_has_field (document, "application_annotations")) {
-		CouchdbStructField *annotations = couchdb_document_get_application_annotations (document);
+	if (app_annotations != NULL) {
+		/* Always have a REV field on the VCARD, for SyncEvolution (bug LP:#479110) */
+		CouchdbStructField *evo_annotations;
+		gchar time_string[100] = {0};
 
-		str = couchdb_struct_field_to_string (annotations);
+		evo_annotations = couchdb_struct_field_get_struct_field (app_annotations, "Evolution");
+		if (evo_annotations != NULL) {
+			if (couchdb_struct_field_has_field (evo_annotations, "revision")) {
+				e_contact_set (contact, E_CONTACT_REV,
+					       couchdb_struct_field_get_string_field (evo_annotations, "revision"));
+			} else {
+				get_current_time (time_string);
+				e_contact_set (contact, E_CONTACT_REV, time_string);
+			}
+		} else {
+			get_current_time (time_string);
+			e_contact_set (contact, E_CONTACT_REV, time_string);
+		}
+
+		/* Save the entire app_annotations field as a string on the VCARD */
+		str = couchdb_struct_field_to_string (app_annotations);
 		e_vcard_add_attribute_with_value (E_VCARD (contact),
 						  e_vcard_attribute_new (NULL, COUCHDB_APPLICATION_ANNOTATIONS_PROP),
 						  str);
 
 		g_free (str);
-		couchdb_struct_field_unref (annotations);
+		couchdb_struct_field_unref (app_annotations);
+	} else {
+		/* Always have a REV field on the VCARD, for SyncEvolution (bug LP:#479110) */
+		gchar time_string[100] = {0};
+
+		get_current_time (time_string);
+		e_contact_set (contact, E_CONTACT_REV, time_string);
 	}
 
 	/* convert the contact to a VCARD string to be returned */
@@ -456,7 +501,7 @@ contact_email_to_struct_field (EVCardAttribute *attr)
 
 	params = e_vcard_attribute_get_params (attr);
 	if (!params)
-		return couchdb_document_contact_email_new (NULL, email, NULL);
+		return desktopcouch_document_contact_email_new (NULL, email, NULL);
 
 	for (pl = params; pl != NULL; pl = pl->next) {
 		GList *v;
@@ -481,7 +526,7 @@ contact_email_to_struct_field (EVCardAttribute *attr)
 		}
 	}
 
-	return couchdb_document_contact_email_new (uuid, email, description);
+	return desktopcouch_document_contact_email_new (uuid, email, description);
 }
 
 static CouchdbStructField *
@@ -499,7 +544,7 @@ contact_phone_to_struct_field (EVCardAttribute *attr)
 
 	params = e_vcard_attribute_get_params (attr);
 	if (!params)
-		return couchdb_document_contact_phone_new (NULL, phone, NULL, /* FIXME */ 0);
+		return desktopcouch_document_contact_phone_new (NULL, phone, NULL, /* FIXME */ 0);
 
 	for (pl = params; pl != NULL; pl = pl->next) {
 		GList *v;
@@ -548,7 +593,7 @@ contact_phone_to_struct_field (EVCardAttribute *attr)
 	else
 		final_description = g_strdup_printf ("%s %s", description, kind);
 
-	sf = couchdb_document_contact_phone_new (uuid, phone, final_description, /* FIXME */ 0);
+	sf = desktopcouch_document_contact_phone_new (uuid, phone, final_description, /* FIXME */ 0);
 	g_free (final_description);
 
 	return sf;
@@ -569,7 +614,7 @@ contact_postal_address_to_struct_field (EContact *contact, EContactField field,
 		else if (contact_address->street && *contact_address->street)
 			street = g_strdup (contact_address->street);
 
-		sf = couchdb_document_contact_address_new (/* FIXME */ NULL,
+		sf = desktopcouch_document_contact_address_new (/* FIXME */ NULL,
 							   street,
 							   contact_address->locality,
 							   contact_address->region,
@@ -597,7 +642,7 @@ contact_url_to_struct_field (EVCardAttribute *attr, const gchar *description)
 
 	params = e_vcard_attribute_get_params (attr);
 	if (!params)
-		return couchdb_document_contact_url_new (NULL, address, description);
+		return desktopcouch_document_contact_url_new (NULL, address, description);
 
 	for (pl = params; pl != NULL; pl = pl->next) {
 		GList *v;
@@ -610,7 +655,7 @@ contact_url_to_struct_field (EVCardAttribute *attr, const gchar *description)
 		}
 	}
 
-	return couchdb_document_contact_url_new (uuid, address, description);
+	return desktopcouch_document_contact_url_new (uuid, address, description);
 }
 
 static CouchdbStructField *
@@ -625,7 +670,7 @@ contact_im_to_struct_field (EVCardAttribute *attr, const gchar *protocol)
 
 	params = e_vcard_attribute_get_params (attr);
 	if (!params)
-		return couchdb_document_contact_im_new (NULL, address, "other", protocol);
+		return desktopcouch_document_contact_im_new (NULL, address, "other", protocol);
 
 	for (pl = params; pl != NULL; pl = pl->next) {
 		GList *v;
@@ -648,7 +693,32 @@ contact_im_to_struct_field (EVCardAttribute *attr, const gchar *protocol)
 		}
 	}
 
-	return couchdb_document_contact_im_new (uuid, address, description, protocol);
+	return desktopcouch_document_contact_im_new (uuid, address, description, protocol);
+}
+
+static void
+set_vcard_revision (CouchdbStructField *app_annotations, EContact *contact)
+{
+	CouchdbStructField *evo_annotations;
+	const gchar *rev;
+
+	if (couchdb_struct_field_has_field (app_annotations, "Evolution"))
+		evo_annotations = couchdb_struct_field_get_struct_field (app_annotations, "Evolution");
+	else
+		evo_annotations = couchdb_struct_field_new ();
+
+	rev = e_contact_get_const (contact,  E_CONTACT_REV);
+	if (rev && *rev) {
+		couchdb_struct_field_set_string_field (evo_annotations, "revision", rev);
+	} else { 
+		gchar time_string[100] = {0};
+
+		get_current_time (time_string);
+		couchdb_struct_field_set_string_field (evo_annotations, "revision", time_string);
+	}
+
+	couchdb_struct_field_set_struct_field (app_annotations, "Evolution", evo_annotations);
+	couchdb_struct_field_unref (evo_annotations);
 }
 
 static CouchdbDocument *
@@ -660,10 +730,11 @@ couch_document_from_contact (EBookBackendCouchDB *couchdb_backend, EContact *con
 	const char *str;
 	CouchdbDocument *document;
 	gint i;
-	CouchdbStructField *postal_address;
+	CouchdbStructField *postal_address, *app_annotations;
+	EContactName *contact_name;
 
 	/* create the CouchDBDocument to put on the database */
-	document = couchdb_document_contact_new (couchdb_backend->couchdb);
+	document = desktopcouch_document_contact_new (couchdb_backend->couchdb);
 
 	str = e_contact_get_const (contact, E_CONTACT_UID);
 	if (str)
@@ -673,37 +744,54 @@ couch_document_from_contact (EBookBackendCouchDB *couchdb_backend, EContact *con
 	if (str)
 		couchdb_document_set_revision (document, str);
 
-	couchdb_document_contact_set_first_name (document, (const char *) e_contact_get_const (contact, E_CONTACT_GIVEN_NAME));
-	couchdb_document_contact_set_last_name (document, (const char *) e_contact_get_const (contact, E_CONTACT_FAMILY_NAME));
-	couchdb_document_contact_set_nick_name (document, (const char *) e_contact_get_const (contact, E_CONTACT_NICKNAME));
-	couchdb_document_contact_set_spouse_name (document, (const char *) e_contact_get_const (contact, E_CONTACT_SPOUSE));
+	contact_name = (EContactName *) e_contact_get (contact, E_CONTACT_NAME);
+	if (contact_name != NULL) {
+		if (contact_name->prefixes != NULL)
+			desktopcouch_document_contact_set_title (document, (const char *) contact_name->prefixes);
+		if (contact_name->given != NULL)
+			desktopcouch_document_contact_set_first_name (document, (const char *) contact_name->given);
+		if (contact_name->additional != NULL)
+			desktopcouch_document_contact_set_middle_name (document, (const gchar *) contact_name->additional);
+		if (contact_name->family != NULL)
+			desktopcouch_document_contact_set_last_name (document, (const char *) contact_name->family);
+		if (contact_name->suffixes != NULL)
+			desktopcouch_document_contact_set_suffix (document, (const char *) contact_name->suffixes);
+	}
 
-	couchdb_document_contact_set_company (document, (const char *) e_contact_get_const (contact, E_CONTACT_ORG));
-	couchdb_document_contact_set_department (document, (const char *) e_contact_get_const (contact, E_CONTACT_ORG_UNIT));
-	couchdb_document_contact_set_title (document, (const char *) e_contact_get_const (contact, E_CONTACT_TITLE));
-	couchdb_document_contact_set_job_title (document, (const char *) e_contact_get_const (contact, E_CONTACT_ROLE));
-	couchdb_document_contact_set_manager_name (document, (const char *) e_contact_get_const (contact, E_CONTACT_MANAGER));
-	couchdb_document_contact_set_assistant_name (document, (const char *) e_contact_get_const (contact, E_CONTACT_ASSISTANT));
-	couchdb_document_contact_set_office (document, (const char *) e_contact_get_const (contact, E_CONTACT_OFFICE));
-	couchdb_document_contact_set_notes (document, (const char *) e_contact_get_const (contact, E_CONTACT_NOTE));
+	desktopcouch_document_contact_set_nick_name (document, (const char *) e_contact_get_const (contact, E_CONTACT_NICKNAME));
+	desktopcouch_document_contact_set_spouse_name (document, (const char *) e_contact_get_const (contact, E_CONTACT_SPOUSE));
+
+	desktopcouch_document_contact_set_company (document, (const char *) e_contact_get_const (contact, E_CONTACT_ORG));
+	desktopcouch_document_contact_set_department (document, (const char *) e_contact_get_const (contact, E_CONTACT_ORG_UNIT));
+	desktopcouch_document_contact_set_job_title (document, (const char *) e_contact_get_const (contact, E_CONTACT_TITLE));
+	desktopcouch_document_contact_set_manager_name (document, (const char *) e_contact_get_const (contact, E_CONTACT_MANAGER));
+	desktopcouch_document_contact_set_assistant_name (document, (const char *) e_contact_get_const (contact, E_CONTACT_ASSISTANT));
+	desktopcouch_document_contact_set_office (document, (const char *) e_contact_get_const (contact, E_CONTACT_OFFICE));
+	desktopcouch_document_contact_set_categories (document, (const char *) e_contact_get_const (contact, E_CONTACT_CATEGORIES));
+	desktopcouch_document_contact_set_notes (document, (const char *) e_contact_get_const (contact, E_CONTACT_NOTE));
 
 	/* email addresses */
 	list = NULL;
 	attr_list = e_contact_get_attributes (contact, E_CONTACT_EMAIL);
-	for (al = attr_list; al != NULL; al = al->next) {
-		CouchdbStructField *sf;
-		EVCardAttribute *attr = (EVCardAttribute *) al->data;
+	if (attr_list != NULL) {
+		for (al = attr_list; al != NULL; al = al->next) {
+			CouchdbStructField *sf;
+			EVCardAttribute *attr = (EVCardAttribute *) al->data;
 
-		sf = contact_email_to_struct_field (attr);
-		if (sf)
-			list = g_slist_append (list, sf);
-	}
+			sf = contact_email_to_struct_field (attr);
+			if (sf)
+				list = g_slist_append (list, sf);
+		}
 
-	if (list) {
-		couchdb_document_contact_set_email_addresses (document, list);
+		if (list) {
+			desktopcouch_document_contact_set_email_addresses (document, list);
 
-		g_slist_foreach (list, (GFunc) couchdb_struct_field_unref, NULL);
-		g_slist_free (list);
+			g_slist_foreach (list, (GFunc) couchdb_struct_field_unref, NULL);
+			g_slist_free (list);
+		}
+
+		g_list_foreach (attr_list, (GFunc) e_vcard_attribute_free, NULL);
+		g_list_free (attr_list);
 	}
 
 	/* phone numbers */
@@ -721,7 +809,7 @@ couch_document_from_contact (EBookBackendCouchDB *couchdb_backend, EContact *con
 	}
 
 	if (list) {
-		couchdb_document_contact_set_phone_numbers (document, list);
+		desktopcouch_document_contact_set_phone_numbers (document, list);
 
 		g_slist_foreach (list, (GFunc) couchdb_struct_field_unref, NULL);
 		g_slist_free (list);
@@ -742,7 +830,7 @@ couch_document_from_contact (EBookBackendCouchDB *couchdb_backend, EContact *con
 		list = g_slist_append (list, postal_address);
 
 	if (list) {
-		couchdb_document_contact_set_addresses (document, list);
+		desktopcouch_document_contact_set_addresses (document, list);
 
 		g_slist_foreach (list, (GFunc) couchdb_struct_field_unref, NULL);
 		g_slist_free (list);
@@ -767,7 +855,7 @@ couch_document_from_contact (EBookBackendCouchDB *couchdb_backend, EContact *con
 	}
 
 	if (list != NULL) {
-		couchdb_document_contact_set_urls (document, list);
+		desktopcouch_document_contact_set_urls (document, list);
 
 		g_slist_foreach (list, (GFunc) couchdb_struct_field_unref, NULL);
 		g_slist_free (list);
@@ -782,35 +870,35 @@ couch_document_from_contact (EBookBackendCouchDB *couchdb_backend, EContact *con
 
 		if (g_strcmp0 (e_vcard_attribute_get_name (attr),
 			       e_contact_vcard_attribute (E_CONTACT_IM_AIM)) == 0)
-			sf = contact_im_to_struct_field (attr, COUCHDB_DOCUMENT_CONTACT_IM_PROTOCOL_AIM);
+			sf = contact_im_to_struct_field (attr, DESKTOPCOUCH_DOCUMENT_CONTACT_IM_PROTOCOL_AIM);
 		else if (g_strcmp0 (e_vcard_attribute_get_name (attr),
 			       e_contact_vcard_attribute (E_CONTACT_IM_GADUGADU)) == 0)
-			sf = contact_im_to_struct_field (attr, COUCHDB_DOCUMENT_CONTACT_IM_PROTOCOL_GADU_GADU);
+			sf = contact_im_to_struct_field (attr, DESKTOPCOUCH_DOCUMENT_CONTACT_IM_PROTOCOL_GADU_GADU);
 		else if (g_strcmp0 (e_vcard_attribute_get_name (attr),
 			       e_contact_vcard_attribute (E_CONTACT_IM_GROUPWISE)) == 0)
-			sf = contact_im_to_struct_field (attr, COUCHDB_DOCUMENT_CONTACT_IM_PROTOCOL_GROUPWISE);
+			sf = contact_im_to_struct_field (attr, DESKTOPCOUCH_DOCUMENT_CONTACT_IM_PROTOCOL_GROUPWISE);
 		else if (g_strcmp0 (e_vcard_attribute_get_name (attr),
 			       e_contact_vcard_attribute (E_CONTACT_IM_ICQ)) == 0)
-			sf = contact_im_to_struct_field (attr, COUCHDB_DOCUMENT_CONTACT_IM_PROTOCOL_ICQ);
+			sf = contact_im_to_struct_field (attr, DESKTOPCOUCH_DOCUMENT_CONTACT_IM_PROTOCOL_ICQ);
 		else if (g_strcmp0 (e_vcard_attribute_get_name (attr),
 			       e_contact_vcard_attribute (E_CONTACT_IM_JABBER)) == 0)
-			sf = contact_im_to_struct_field (attr, COUCHDB_DOCUMENT_CONTACT_IM_PROTOCOL_JABBER);
+			sf = contact_im_to_struct_field (attr, DESKTOPCOUCH_DOCUMENT_CONTACT_IM_PROTOCOL_JABBER);
 		else if (g_strcmp0 (e_vcard_attribute_get_name (attr),
 			       e_contact_vcard_attribute (E_CONTACT_IM_MSN)) == 0)
-			sf = contact_im_to_struct_field (attr, COUCHDB_DOCUMENT_CONTACT_IM_PROTOCOL_MSN);
+			sf = contact_im_to_struct_field (attr, DESKTOPCOUCH_DOCUMENT_CONTACT_IM_PROTOCOL_MSN);
 		else if (g_strcmp0 (e_vcard_attribute_get_name (attr),
 			       e_contact_vcard_attribute (E_CONTACT_IM_SKYPE)) == 0)
-			sf = contact_im_to_struct_field (attr, COUCHDB_DOCUMENT_CONTACT_IM_PROTOCOL_SKYPE);
+			sf = contact_im_to_struct_field (attr, DESKTOPCOUCH_DOCUMENT_CONTACT_IM_PROTOCOL_SKYPE);
 		else if (g_strcmp0 (e_vcard_attribute_get_name (attr),
 			       e_contact_vcard_attribute (E_CONTACT_IM_YAHOO)) == 0)
-			sf = contact_im_to_struct_field (attr, COUCHDB_DOCUMENT_CONTACT_IM_PROTOCOL_YAHOO);
+			sf = contact_im_to_struct_field (attr, DESKTOPCOUCH_DOCUMENT_CONTACT_IM_PROTOCOL_YAHOO);
 
 		if (sf != NULL)
 			list = g_slist_append (list, sf);
 	}
 
 	if (list != NULL) {
-		couchdb_document_contact_set_im_addresses (document, list);
+		desktopcouch_document_contact_set_im_addresses (document, list);
 
 		g_slist_foreach (list, (GFunc) couchdb_struct_field_unref, NULL);
 		g_slist_free (list);
@@ -821,7 +909,7 @@ couch_document_from_contact (EBookBackendCouchDB *couchdb_backend, EContact *con
 	if (dt) {
 		char *dt_str = e_contact_date_to_string (dt);
 		if (dt_str != NULL) {
-			couchdb_document_contact_set_birth_date (document, (const char *) dt_str);
+			desktopcouch_document_contact_set_birth_date (document, (const char *) dt_str);
 
 			g_free (dt_str);
 		}
@@ -832,7 +920,7 @@ couch_document_from_contact (EBookBackendCouchDB *couchdb_backend, EContact *con
 	if (dt) {
 		char *dt_str = e_contact_date_to_string (dt);
 		if (dt_str != NULL) {
-			couchdb_document_contact_set_wedding_date (document, (const char *) dt_str);
+			desktopcouch_document_contact_set_wedding_date (document, (const char *) dt_str);
 
 			g_free (dt_str);
 		}
@@ -840,15 +928,14 @@ couch_document_from_contact (EBookBackendCouchDB *couchdb_backend, EContact *con
 
 	/* application annotations */
 	str = e_vcard_attribute_get_value (e_vcard_get_attribute (E_VCARD (contact), COUCHDB_APPLICATION_ANNOTATIONS_PROP));
-	if (str) {
-		CouchdbStructField *annotations;
+	if (str)
+		app_annotations = couchdb_struct_field_new_from_string (str);
+	else
+		app_annotations = couchdb_struct_field_new ();
 
-		annotations = couchdb_struct_field_new_from_string (str);
-		if (annotations) {
-			couchdb_document_set_application_annotations (document, annotations);
-			couchdb_struct_field_unref (annotations);
-		}
-	}
+	set_vcard_revision (app_annotations, contact);
+	desktopcouch_document_set_application_annotations (document, app_annotations);
+	couchdb_struct_field_unref (app_annotations);
 
 	return document;
 }
@@ -1071,7 +1158,7 @@ e_book_backend_couchdb_remove_contacts (EBookBackend *backend,
 
 				/* For desktopcouch, we don't remove contacts, we just
 				 * mark them as deleted */
-				app_annotations = couchdb_document_get_application_annotations (document);
+				app_annotations = desktopcouch_document_get_application_annotations (document);
 				if (app_annotations == NULL)
 					app_annotations = couchdb_struct_field_new ();
 
@@ -1086,7 +1173,7 @@ e_book_backend_couchdb_remove_contacts (EBookBackend *backend,
 				couchdb_struct_field_set_boolean_field (private_annotations, "deleted", TRUE);
 				couchdb_struct_field_set_struct_field (u1_annotations, "private_application_annotations", private_annotations);
 				couchdb_struct_field_set_struct_field (app_annotations, "Ubuntu One", u1_annotations);
-				couchdb_document_set_application_annotations (document, app_annotations);
+				desktopcouch_document_set_application_annotations (document, app_annotations);
 
 				/* Now put the new revision of the document */
 				if (couchdb_document_put (document, couchdb_backend->dbname, &error))
@@ -1353,11 +1440,14 @@ e_book_backend_couchdb_get_supported_fields (EBookBackend *backend,
 	fields = g_list_append (fields, g_strdup (e_contact_field_name (E_CONTACT_GIVEN_NAME)));
 	fields = g_list_append (fields, g_strdup (e_contact_field_name (E_CONTACT_FAMILY_NAME)));
 	fields = g_list_append (fields, g_strdup (e_contact_field_name (E_CONTACT_FULL_NAME)));
+	fields = g_list_append (fields, g_strdup (e_contact_field_name (E_CONTACT_NAME)));
 	fields = g_list_append (fields, g_strdup (e_contact_field_name (E_CONTACT_NICKNAME)));
 	fields = g_list_append (fields, g_strdup (e_contact_field_name (E_CONTACT_SPOUSE)));
 	fields = g_list_append (fields, g_strdup (e_contact_field_name (E_CONTACT_BIRTH_DATE)));
 	fields = g_list_append (fields, g_strdup (e_contact_field_name (E_CONTACT_ANNIVERSARY)));
 	fields = g_list_append (fields, g_strdup (e_contact_field_name (E_CONTACT_NOTE)));
+	fields = g_list_append (fields, g_strdup (e_contact_field_name (E_CONTACT_CATEGORIES)));
+	fields = g_list_append (fields, g_strdup (e_contact_field_name (E_CONTACT_REV)));
 
 	/* URLS */
 	fields = g_list_append (fields, g_strdup (e_contact_field_name (E_CONTACT_HOMEPAGE_URL)));
@@ -1367,7 +1457,6 @@ e_book_backend_couchdb_get_supported_fields (EBookBackend *backend,
 	fields = g_list_append (fields, g_strdup (e_contact_field_name (E_CONTACT_ORG)));
 	fields = g_list_append (fields, g_strdup (e_contact_field_name (E_CONTACT_ORG_UNIT)));
 	fields = g_list_append (fields, g_strdup (e_contact_field_name (E_CONTACT_TITLE)));
-	fields = g_list_append (fields, g_strdup (e_contact_field_name (E_CONTACT_ROLE)));
 	fields = g_list_append (fields, g_strdup (e_contact_field_name (E_CONTACT_MANAGER)));
 	fields = g_list_append (fields, g_strdup (e_contact_field_name (E_CONTACT_ASSISTANT)));
 	fields = g_list_append (fields, g_strdup (e_contact_field_name (E_CONTACT_OFFICE)));
